@@ -36,14 +36,13 @@ from .exceptions import ValidationError, NotFoundError, DatabaseError
 from leveldb import WriteBatch, LevelDB
 
 class EmDocumentMetaclass(type):
-  def __new__(cls, clsname, parents, attrs, build_indexes=False):
+  def __new__(cls, clsname, parents, attrs):
     if clsname in ("Document", "EmDocument"):
       return type.__new__(cls, clsname, parents, attrs)
 
     meta = {}
 
-    if build_indexes:
-      indexes = []
+    indexes = []
 
     all_parents = reversed(walk_parents(parents))
 
@@ -51,16 +50,18 @@ class EmDocumentMetaclass(type):
       if hasattr(p_cls, "_meta"):
         meta.update(p_cls._meta)
 
+      if hasattr(p_cls, "_indexes"):
+        indexes += list(p_cls._indexes)
+
     for name in attrs.keys():
       if isinstance(attrs[name], BaseProperty):
         meta[name] = attrs.pop(name)
-        if build_indexes and isinstance(meta[name], (StringProperty, NumberProperty, ListProperty, ReferenceProperty)) and meta[name]._index:
+        if isinstance(meta[name], (StringProperty, NumberProperty, ListProperty, ReferenceProperty)) and meta[name]._index:
           indexes.append(name)
 
     attrs["_meta"] = meta
     attrs["defined_properties"] = meta.keys()
-    if build_indexes:
-      attrs["_indexes"] = indexes
+    attrs["_indexes"] = indexes
     return type.__new__(cls, clsname, parents, attrs)
 
   def __getattr__(self, name):
@@ -284,7 +285,7 @@ class EmDocument(object):
       name: name of the attribute
       value: the value of that attribute to set to.
     """
-    if name[0] == "_":
+    if name[0] == "_" or name == "key":
       self.__dict__[name] = value
       return
 
@@ -330,10 +331,9 @@ class DocumentMetaclass(EmDocumentMetaclass):
     attrs["_write_batch"] = WriteBatch()
     attrs["_index_write_needed"] = False
 
-    if "indexdb" in attrs and attrs["indexdb"] is not None:
-      attrs["_indexdb_write_batch"] = WriteBatch()
+    attrs["_indexdb_write_batch"] = WriteBatch()
 
-    return EmDocumentMetaclass.__new__(cls, clsname, parents, attrs, True)
+    return EmDocumentMetaclass.__new__(cls, clsname, parents, attrs)
 
 _INDEX_KEY = "{f}~{v}"
 
@@ -380,6 +380,7 @@ class Document(EmDocument):
   def _flush_indexes(cls, sync=True):
     if cls._index_write_needed:
       cls._get_indexdb().Write(cls._indexdb_write_batch, sync=sync)
+      cls._indexdb_write_batch = WriteBatch()
       cls._index_write_needed = False
 
   @classmethod
