@@ -62,9 +62,11 @@ class DocumentLater(Document):
 
 class Mixin(EmDocument):
   test = StringProperty(validators=lambda v: v == "test")
+  test_index = StringProperty(index=True)
 
 class DocumentWithMixin(Document, Mixin):
   db = leveldb.LevelDB("{0}/mixin.db".format(test_dir))
+  indexdb = leveldb.LevelDB("{0}/test_mixin_index.db".format(test_dir))
 
 class BasicDocumentTest(unittest.TestCase):
   def setUp(self):
@@ -121,6 +123,16 @@ class BasicDocumentTest(unittest.TestCase):
     with self.assertRaises(NotFoundError):
       SomeDocument.get(doc.key)
 
+  def test_document_mixin_indexes_inheritance(self):
+    doc = DocumentWithMixin()
+    doc.test = "test"
+    doc.test_index = "a"
+    doc.save()
+    self.cleanups.append(doc)
+    keys = DocumentWithMixin.index_keys_only("test_index", "a")
+    self.assertEquals(1, len(keys))
+    self.assertEquals(doc.key, keys[0])
+
   def test_document_mixin(self):
     doc = DocumentWithMixin()
     doc.test = "test"
@@ -323,6 +335,13 @@ class BasicDocumentTest(unittest.TestCase):
 
     self.assertEquals(5, i)
 
+  def test_set_key(self):
+    doc = SomeDocument()
+    k = doc.key
+    doc.key = "hello"
+    self.assertNotEquals(k, doc.key)
+    self.assertEquals("hello", doc.key)
+
   def test_index_buckets(self):
     SomeDocument("1").save()
     SomeDocument("2").save()
@@ -348,4 +367,4 @@ class BasicDocumentTest(unittest.TestCase):
     self.assertEquals(6, i)
 
 if __name__ == "__main__":
-  unittest.main()
\ No newline at end of file
+  unittest.main()
